/*
 * AppInfoUtil.java
 *
 * Created on 07 July 2006, 10:45
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package jim.common;

import javax.servlet.ServletContext;
import org.apache.commons.logging.Log;

/**
 * Implements access to a singleton AppInfo object stored in the ServletContext
 * @author briggsj
 */
public class AppInfoUtil {
    
    static private Log log = Logger.getLog(AppInfoUtil.class);
    
    /** Creates a new instance of AppInfoUtil */
    public AppInfoUtil() {
    }
    
    static private final String APPINFO = "APPINFO";
    static private ServletContext myServletContext = null;
    
    /**
     * Initialise by specifying the servlet context
     * @param sc the webapp's servlet context
     */
    public static void init(ServletContext sc) {
        log.info("initialising AppInfo object in servlet context");
        myServletContext = sc;
        
        AppInfo appinfo = getAppInfo();
        if (appinfo == null) {
            log.info("creating new AppInfo object");
            appinfo = new AppInfo();
            sc.setAttribute(APPINFO, appinfo);
        }
    }
    
    private static AppInfo getAppInfo() {
        log.debug("accessing AppInfo");
        return (AppInfo)myServletContext.getAttribute(APPINFO);
    }
    
    /**
     * Utility method for setting a parameter within the global AppInfo
     * @param name parameter name
     * @param value parameter value
     */
    static public void setParameter(String name, Object value) {
        log.debug("set AppInfo parameter '" + name + "' to '" + value + "'");
        synchronized(myServletContext) {
            AppInfo info = getAppInfo();
            info.put(name, value);
        }
    }
    
    static public Object getParameter(String name) {
        Object value = getAppInfo().get(name);
        log.debug("got AppInfo parameter '" + name + "'='" + value + "'");
        return value;
    }
}
