/*
 * ComposeAction.java
 *
 * Created on 24 April 2005, 10:00
 */

package jim.mailapp.struts;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;

import jim.common.JimFatalException;
import jim.mailapp.MessageProcess;
import jim.util.struts.JimActionErrors;
import jim.util.struts.StrutsUtils;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.struts.action.ActionForward;

/**
 *
 * @author  briggsj
 */
public class ComposeAction extends MailAppBaseAction {
    
    /** Creates a new instance of ComposeAction */
    public ComposeAction() {
    }
    
    private ComposeForm open(String messageName, MessageProcess mp, StrutsUtils su) {
        JimActionErrors errs = su.getErrors();
        JimActionErrors msgs = su.getMessages();
        
        ComposeForm outf = new ComposeForm();
        msgs.addGlobal("compose.open", messageName);
        outf.setMessageName(messageName);
        outf.setOpenMessageName(messageName);
        
        if (!"".equals(messageName)) {
            Map msgmap = mp.getMessages();
            if (msgmap.containsKey(messageName)) {
                String editText = (String)msgmap.get(messageName);
                outf.setEditText(editText);
                
                String text = mp.getEmailMessage(messageName).getMessage();
                outf.setText(text);
                outf.setChanged(false);
            } else {
                errs.addGlobal("compose.nosuchmessage", messageName);
                return null;
            }
        } else {
            outf.setEditText("");
            outf.setText("");
            outf.setChanged(false);
        }
        return outf;
    }
    
    private ComposeForm save(String messageName, ComposeForm inf, MessageProcess mp, StrutsUtils su) {
        ComposeForm outf = new ComposeForm();
        try {
            BeanUtils.copyProperties(outf, inf);
        } catch (Exception e) {
            outf = inf;
        }
        boolean ok = true;
        
        if (ok) {
            mp.setMessage(messageName, inf.getEditText());
            outf.setChanged(false);
            String text = mp.getEmailMessage(messageName).getMessage();
            outf.setText(text);
            outf.setMessageName(messageName);
            outf.setSaveMessageName("");
            outf.setOpenMessageName(messageName);
        }
        return outf;
    }
    
    private boolean discardable(ComposeForm inf, MessageProcess mp, StrutsUtils su) {
        if (inf.isDiscard()) {
            return true;
        }
        boolean changed = changed(inf, mp);
        return changed;
    }
    
    /**
     * @param inf
     * @param mp
     * @return
     */
    private boolean changed(ComposeForm inf, MessageProcess mp) {
        String original;
        if (inf.getMessageName() != null && !"".equals(inf.getMessageName())) {
            original = mp.getEmailMessage(inf.getMessageName()).getText();
        } else {
            original = "";
        }
        String modified = inf.getEditText();
        boolean changed = original.equals(modified);
        inf.setChanged(changed);
        return changed;
    }
    
    public ActionForward doExecute(StrutsUtils su) throws IOException, ServletException, JimFatalException {
        ComposeForm inf = (ComposeForm)su.getForm();
        ComposeForm outf = inf;
        String button = inf.getButton();
        MessageProcess mp;
        try {
            mp = new MessageProcess();
        } catch (Exception e) {
            throw new JimFatalException("Cannot open message pool");
        }
        
        //dispatch on value of "button"
        if ("Open...".equalsIgnoreCase(button)) {
            if (discardable(inf, mp, su)) {
                outf = open(inf.getOpenMessageName(), mp, su);
            } else {
                JimActionErrors errs = su.getErrors();
                errs.add("discard", "compose.discard", inf.getOpenMessageName());
            }
        } else if ("Save".equalsIgnoreCase(button)) {
            outf = save(inf.getMessageName(), inf, mp, su);
        } else if ("Save as...".equalsIgnoreCase(button)) {
            outf = save(inf.getSaveMessageName(), inf, mp, su);
        } else if ("Send".equalsIgnoreCase(button)) {
            
        } else {
            outf = open("", mp, su);
        }
        su.setAttribute("composeForm", outf, StrutsUtils.REQUEST_SCOPE);
        su.setAttribute("compose.messages", mp.makeComposeMessageBean(), StrutsUtils.REQUEST_SCOPE);
        setCourses(su, mp);
        return su.successOrFailure();
    }
    
    private void setCourses (StrutsUtils su, MessageProcess mp) {
        HttpSession session = su.getRequest().getSession();
        List courses = (List)session.getAttribute("Courses");
        if (courses == null) {
            try { 
            courses = mp.getCourses("ey2005");
            } catch (java.sql.SQLException e) {
                su.getErrors().addException(e);
            }
            su.setAttribute("Courses", courses, StrutsUtils.SESSION_SCOPE);
        }
    }
}
