/*
 * DateFormatter.java
 *
 * Created on 24 July 2005, 17:00
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package jim.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Used for date/time formatting to make insertions into MySQL
 * @author Steve Powell
 */
public class DateFormatter {
    
    /**
     * Preformatted date string
     */
    private String origDateStr = "";
    /**
     * Date format dd/mm/yyyy
     */
    private static final SimpleDateFormat NICE_DATE_FORMAT = new SimpleDateFormat("dd/MM/yyyy");  
    /**
     * Date format yyyy/mm/dd
     */
    private static final SimpleDateFormat SQL_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd");
    /**
     * Time format hh:mm
     */
    private static final SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("hh:mm:ss");

        
    /** Creates a new instance of DateFormatter */
    public DateFormatter() {       
        
    }
    
    /**
     * Creates a new instance of DateFormatter with initial date string
     * @param origDateStr Date string to be formatted
     */
    public DateFormatter(String origDateStr) {
        
        this.origDateStr = origDateStr;
        
    }
    
    /**
     * Creates a new instance of DateFormatter with initial date
     * @param origDate Date to be formatted
     */
    public DateFormatter(Date origDate) {       
              
        this.origDateStr = NICE_DATE_FORMAT.format(origDate);
        
    }
    
    /**
     * Applies formatting to date
     * @return Formatted date
     * @throws java.text.ParseException As a result of parsing error
     */
    public String format() throws ParseException {
       
//        Date date = new Date();
        Date date = NICE_DATE_FORMAT.parse(origDateStr);
        return SQL_DATE_FORMAT.format(date);
        
    }
    
    public java.sql.Date toDate() throws ParseException {
        return java.sql.Date.valueOf(format());
    }
    
    /**
     * Applies formatting to time
     * @param origDate Time to be formatted
     * @return Formatted time
     * @throws java.text.ParseException As a result of parsing error
     */
    public String timeformat(Date origDate) throws ParseException {
        return TIME_FORMAT.format(origDate).toString();                      
    }
    
}
