/*
 * SessionFilter.java
 *
 * Created on 01 April 2005, 21:26
 */

package jim.util;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;


/**
 * A filter to monitor the memory usage of the Runtime
 * @author briggsj
 */
public class MemoryFilter implements Filter {
    
    static Logger logger = Logger.getLogger(MemoryFilter.class);
    static Runtime rt = Runtime.getRuntime();
    static long warnFree = 0;
    
    /** Creates a new instance of SessionFilter */
    public MemoryFilter() {
        super();
    }
    
    /**
     * Destroy object
     */
    public void destroy() {
    }
    
    /**
     * Implement the session filter
     * @param servletRequest Request
     * @param response Response
     * @param chain Chain
     * @throws IOException exception
     * @throws ServletException exception
     */
    public void doFilter(ServletRequest servletRequest,
            ServletResponse response,
            FilterChain chain) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest)servletRequest;
        
        long maxMemory0 = rt.maxMemory();
        long freeMemory0 = rt.freeMemory();
        long totalMemory0 = rt.totalMemory();
        
        if (freeMemory0 < warnFree) {
            logger.warn("free memory down to " + freeMemory0);
        }
        
        chain.doFilter(request, response);
        
        long maxMemory1 = rt.maxMemory();
        long freeMemory1 = rt.freeMemory();
        long totalMemory1 = rt.totalMemory();
        
        if (logger.isInfoEnabled()) {
            logger.info("max0: " + maxMemory0 + " max1: " + maxMemory1
                    + " fre0: " + freeMemory0 + " fre1: " + freeMemory1
                    + " tot0: " + totalMemory0 + " tot1: " + totalMemory1);
        }
    }
    
    /**
     * Initialise the session filter
     * If the initialisation parameter "warnFree" is set, it is used as the threshold for warning about free memory
     * @param cfg the configuration object
     * @throws ServletException exception
     */
    public void init(FilterConfig cfg) throws ServletException {
        String warnFreeAsString = cfg.getInitParameter("warnFree");
        if (warnFreeAsString != null) {
            warnFree = Long.parseLong(warnFreeAsString);
        }
    }
    
  
    
}
