/*
 * MultiDataSource.java
 *
 * Created on 05 June 2006, 15:27
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package jim.util;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.sql.DataSource;
import jim.common.Logger;
import jim.common.Trace;
import org.apache.commons.logging.Log;

/**
 *
 * @author briggsj
 */
public class MultiDataSource implements DataSource {
    
    static private Log log = Logger.getLog(MultiDataSource.class);
    
    private DataSource masterDs = null;
    
    public DataSource getDataSource() throws SQLException {
        if (masterDs == null) {
            int num = 0;
            Context envCtx;
            try {
                envCtx = NamingUtils.findContext();
            } catch (NamingException e) {
                throw new SQLException("Cannot initialise context");
            }
            do {
                String key = envName + num;
                try {
                    DataSource ds = (DataSource)envCtx.lookup(key);
                    if (ds != null) {
                        try {
                            log.debug("Trying datasource " + key);
                            Connection conn = ds.getConnection();
                            masterDs = ds;
                            conn.close();
                            log.info("Established connection to database " + key);
                            this.setKey(key);
//                            envCtx.addToEnvironment("jdbc/multisource", masterDs);
                            return masterDs;
                        } catch (SQLException e) {
                            log.debug("Could not open " + key + ": " + e.getMessage());
                        }
                    }
                } catch (NamingException e) {
                    Trace.print("Could not open " + key + ": " + e.getMessage());
                }
                
                num++;
            } while (num < 10);
            throw new SQLException("Cannot open any datasources: " + envName);
        } else {
            return masterDs;
        }
       
    }
    
    /** Creates a new instance of MultiDataSource */
    public MultiDataSource() {
    }
    
    /**
     * <p>Sets the maximum time in seconds that this data source will wait
     * while attempting to connect to a database.  A value of zero
     * specifies that the timeout is the default system timeout
     * if there is one; otherwise, it specifies that there is no timeout.
     * When a <code>DataSource</code> object is created, the login timeout is
     * initially zero.
     *
     *
     * @param seconds the data source login time limit
     * @exception SQLException if a database access error occurs.
     * @see #getLoginTimeout
     */
    public void setLoginTimeout(int seconds) throws SQLException {
        getDataSource().setLoginTimeout(seconds);
    }
    
    /**
     * <p>Sets the log writer for this <code>DataSource</code>
     * object to the given <code>java.io.PrintWriter</code> object.
     *
     * <p>The log writer is a character output stream to which all logging
     * and tracing messages for this data source will be
     * printed.  This includes messages printed by the methods of this
     * object, messages printed by methods of other objects manufactured
     * by this object, and so on.  Messages printed to a data source-
     * specific log writer are not printed to the log writer associated
     * with the <code>java.sql.Drivermanager</code> class. When a
     * <code>DataSource</code> object is created the log writer is
     * initially null; in other words, the default is for logging to be
     * disabled.
     *
     *
     * @param out the new log writer; to disable logging, set to null
     * @exception SQLException if a database access error occurs
     * @see #getLogWriter
     */
    public void setLogWriter(PrintWriter out) throws SQLException {
        getDataSource().setLogWriter(out);
    }
    
    /**
     * Gets the maximum time in seconds that this data source can wait
     * while attempting to connect to a database.  A value of zero
     * means that the timeout is the default system timeout
     * if there is one; otherwise, it means that there is no timeout.
     * When a <code>DataSource</code> object is created, the login timeout is
     * initially zero.
     *
     *
     * @return the data source login time limit
     * @exception SQLException if a database access error occurs.
     * @see #setLoginTimeout
     */
    public int getLoginTimeout() throws SQLException {
        return getDataSource().getLoginTimeout();
    }
    
    /**
     * <p>Retrieves the log writer for this <code>DataSource</code>
     * object.
     *
     * <p>The log writer is a character output stream to which all logging
     * and tracing messages for this data source will be
     * printed.  This includes messages printed by the methods of this
     * object, messages printed by methods of other objects manufactured
     * by this object, and so on.  Messages printed to a data source
     * specific log writer are not printed to the log writer associated
     * with the <code>java.sql.Drivermanager</code> class.  When a
     * <code>DataSource</code> object is
     * created, the log writer is initially null; in other words, the
     * default is for logging to be disabled.
     *
     *
     * @return the log writer for this data source or null if
     *        logging is disabled
     * @exception SQLException if a database access error occurs
     * @see #setLogWriter
     */
    public PrintWriter getLogWriter() throws SQLException {
        return this.getDataSource().getLogWriter();
    }
    
    /**
     * <p>Attempts to establish a connection with the data source that
     * this <code>DataSource</code> object represents.
     *
     *
     * @param username the database user on whose behalf the connection is
     *  being made
     * @param password the user's password
     * @return a connection to the data source
     * @exception SQLException if a database access error occurs
     */
    public Connection getConnection(String username, String password) throws SQLException {
        return this.getDataSource().getConnection(username, password);
    }
    
    /**
     * <p>Attempts to establish a connection with the data source that
     * this <code>DataSource</code> object represents.
     *
     *
     * @return a connection to the data source
     * @exception SQLException if a database access error occurs
     */
    public Connection getConnection() throws SQLException {
        return this.getDataSource().getConnection();
    }
    
    /**
     * Holds value of property envName.
     */
    private String envName;
    
    /**
     * Getter for property envName.
     * @return Value of property envName.
     */
    public String getEnvName() {
        return this.envName;
    }
    
    /**
     * Setter for property envName.
     * @param envName New value of property envName.
     */
    public void setEnvName(String envName) {
        this.envName = envName;
    }
    
    /**
     * Holds value of property key.
     */
    private String key;
    
    /**
     * Getter for property key.
     * @return Value of property key.
     */
    public String getKey() {
        return this.key;
    }
    
    /**
     * Setter for property key.
     * @param key New value of property key.
     */
    private void setKey(String key) {
        this.key = key;
    }
    
}
