/*
 * QueryCache.java
 *
 * Created on 02 November 2004, 16:41
 */

package jim.util;

/**
 *
 * @author  briggsj
 */

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class QueryCache {
    
    Connection conn;
    Map<String, PreparedStatement> readable = null;
    Map<String, PreparedStatement> writeable = null;
    static private Map<Connection, QueryCache> conns = null;
    
    /** Creates a new instance of QueryCache */
    private QueryCache(Connection conn) {
        this.conn = conn;
        readable = new HashMap<String, PreparedStatement>();
        writeable = new HashMap<String, PreparedStatement>();
    }
    
    public static QueryCache cacheInstance (Connection conn) {
        if (conns == null) {
            conns = new HashMap<Connection, QueryCache>();
        }
        QueryCache qc = (QueryCache)conns.get(conn);
        if (qc == null) {
            qc = new QueryCache(conn);
            conns.put(conn, qc);
        }
        return qc;
    }
    
    public PreparedStatement getPrep(String sql) throws SQLException {
        if (readable.containsKey(sql)) {
            return readable.get(sql);
        } else {
            PreparedStatement prep = conn.prepareStatement(sql);
            readable.put(sql, prep);
            return prep;
        }
    }
    
    public PreparedStatement getPrepMod(String sql) throws SQLException {
        if (writeable.containsKey(sql)) {
            return writeable.get(sql);
        } else {
            PreparedStatement prep = conn.prepareStatement(sql, ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);
            writeable.put(sql, prep);
            return prep;
        }
    }
    
    private void closedown(Map m) throws Throwable {
        Set keys = m.keySet();
        Iterator i = keys.iterator();
        while (i.hasNext()) {
            String sql = (String)i.next();
            PreparedStatement prepStmt = (PreparedStatement)m.get(sql);
            prepStmt.close();
            m.remove(sql);
        }
    }
    
    protected void finalize() throws Throwable {
        super.finalize();
        closedown(readable);
        closedown(writeable);
    }
}
