/*
 * SessionFilter.java
 *
 * Created on 01 April 2005, 21:26
 */

package jim.util;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;


/**
 * A filter to check whether there is a valid session in existence (i.e. it has not timed out).
 * @author briggsj
 */
public class RemoteUserFilter implements Filter {
    
    /** Creates a new instance of SessionFilter */
    public RemoteUserFilter() {
        super();
    }
    
    /**
     * Destroy object
     */
    public void destroy() {
    }
    
    String badsessionURL = "/pages/badlogin.jsp";
    
    /**
     * Implement the session filter
     * @param servletRequest Request
     * @param response Response
     * @param chain Chain
     * @throws IOException exception
     * @throws ServletException exception
     */
    public void doFilter(ServletRequest servletRequest,
            ServletResponse servletResponse,
            FilterChain chain) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest)servletRequest;
        HttpServletResponse response = (HttpServletResponse)servletResponse;
        HttpSession session = request.getSession(false);
        
        if (isOK(session, request, response)) {
            chain.doFilter(request, response);
        } else {
            RequestDispatcher rd = request.getRequestDispatcher(badsessionURL);
            rd.forward(request, response);
        }
    }
    
    /**
     * Initialise the session filter
     * If the initialisation parameter "forwardPage" is set, it is used as the URL to forward to if the session is invalid
     * @param cfg the configuration object
     * @throws ServletException exception
     */
    public void init(FilterConfig cfg) throws ServletException {
        String forwardPage = cfg.getInitParameter("forwardPage");
        if (forwardPage != null) {
            badsessionURL = forwardPage;
        }
    }
    
    /**
     * Default implementation of method to determine whether a session is valid or not.
     * Always returns false.
     * Suitable to be overridden with a more specific test.
     * @param session session object
     * @param request request object
     * @param response response object
     * @return whether the session is invalid
     */
    protected boolean isOK(HttpSession session, HttpServletRequest request, HttpServletResponse response) {
        String username = request.getRemoteUser();
        session.setAttribute("USERNAME", username);
        return true;
    }
    
}
