/*
 * SQLutils.java
 *
 * Created on 09 March 2004, 11:28
 */

package jim.util;
import java.math.BigDecimal;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.List;
import java.util.Map;
import java.util.Random;

import jim.common.Trace;

import org.apache.commons.dbutils.handlers.MapHandler;
import org.apache.commons.dbutils.handlers.MapListHandler;

/**
 *
 * @author  briggsj
 */
public class SqlUtils {
    
    private Connection conn;
    private QueryCache cache;
    
    /** Creates a new instance of SQLutils */
    public SqlUtils() {
    }
    
    public SqlUtils(Connection conn) {
        this.conn = conn;
        cache = QueryCache.cacheInstance(conn);
    }
    
    static public String literal(String s) {
        s = Regexp.substitute(s, "'", "\'");
        return s;
    }
    
    static public String valid(String s) throws SQLException{
        if (s.matches("[A-Za-z0-9_]+"))
            return s;
        else
            throw new SQLException("Invalid character in string " + s);
    }
    
    static public int nextval(Connection con, String seq) throws SQLException {
        SqlUtils s = new SqlUtils(con);
        return s.nextval(seq);
    }
    
    public int nextval(String seq) throws SQLException {
        String selectStatement = "select " + seq + ".nextval from dual";
        PreparedStatement prepStmt = cache.getPrep(selectStatement);
        
        ResultSet rs = prepStmt.executeQuery();
        int result;
        
        if (rs.next()) {
            result = rs.getInt(1);
            rs.close();
//            prepStmt.close();
        } else {
            rs.close();
//            prepStmt.close();
            throw new SQLException("Cannot generate nextval for sequence " + seq);
        }
        return result;
    }
    
//    /**
//     * Return the number associated with the last insert
//     * @throws java.sql.SQLException in the event of any SQL problem
//     * @return number associated with last insert operation
//     */
//    public int lastval() throws SQLException {
//    // Load the autonumber
//        String select =  "SELECT LAST_INSERT_ID()";
//        ResultSet rs = selectRS(select);
//        int n = -1;
//        while (rs.next()) {
//            n = rs.getInt(1);
//        }
//        rs.close();
//        return n;
//    }
    
    static public String query(String searchString, String fields [], String operator) {
        String words[] = searchString.split("[ ]");
        String match = "(";
        int n = 0;
        
        for (int i = 0; i < words.length; i++) {
            if (words[i].trim().equalsIgnoreCase("and")) {
                operator = "and";
            } else if (words[i].trim().equalsIgnoreCase("or")) {
                operator = "or";
            } else {
                if (n > 0) {
                    match += ")\n" + operator + " (";
                }
                n++;
                for (int j = 0; j < fields.length; j++) {
                    if (j > 0) {
                        match += " or ";
                    }
                    match += "LOWER("+fields[j]+") LIKE LOWER('%"+words[i]+"%')";
                }
            }
        }
        match += ")";
        return match;
    }
    
    static public String inCharSet(String charSet, String [] searchFields) {
        String match = "";
        int n = 0, m = 0;
        
        for (int f = 0; f < searchFields.length; f++) {
            if (n > 0) {
                match += " OR ";
            }
            n++;
            match += searchFields[f] + " IN (";
            m = 0;
            for (int i = 0; i < charSet.length(); i++) {
                if (m > 0) {
                    match += ", ";
                }
                m++;
                match += "'" + charSet.charAt(i) + "'";
            }
            match += ")";
        }
        return match;
    }
    
    private void setParams(PreparedStatement prepStmt, Object[] params) throws SQLException {
        if (params != null) {
            for (int n=0; n<params.length; n++) {
                try {
                    prepStmt.setObject(n+1, params[n]);
                } catch (SQLException e) {
                    SQLException s = new SQLException(e.getMessage() + " at item n=" + n);
                    s.setNextException(e);
                    throw s;
                }
            }
        }
    }
    
    public ResultSet selectRS(String selectStatement, Object[] params) throws SQLException {
        Trace.print("select: " + selectStatement);
        PreparedStatement prepStmt = cache.getPrep(selectStatement);
        setParams(prepStmt, params);
        ResultSet rs = prepStmt.executeQuery();
        return rs;
    }
    
    public ResultSet selectRS(String selectStatement, List params) throws SQLException {
        return selectRS(selectStatement, params.toArray());
    }
    
    public ResultSet selectRS(String selectStatement) throws SQLException {
        return selectRS(selectStatement, (Object[])null);
    }
    
    public ResultSet selectRS(String selectStatement, Object param) throws SQLException  {
        return selectRS(selectStatement, new Object[]{param});
    }
    
    public ResultSet selectRSmod(String selectStatement, Object[] params, boolean lock) throws SQLException {
        Trace.print("select: " + selectStatement);
        if (selectStatement.contains("*")) {
            throw new SQLException("Cannot modify a query that contains *, viz: " + selectStatement);
        }
        if (lock) {
            if (this.getConn().getAutoCommit() == true) {
                throw new SQLException("Should not lock database with AutoCommit set - use transactions");
            }
            selectStatement += " FOR UPDATE";
        }
        PreparedStatement prepStmt = cache.getPrepMod(selectStatement);
        if (prepStmt.getResultSetConcurrency() != ResultSet.CONCUR_UPDATABLE) {
            throw new SQLException("ResultSet not updatable for " + selectStatement);
        }
        setParams(prepStmt, params);
        ResultSet rs = prepStmt.executeQuery();
        return rs;
    }
    
    public ResultSet selectRSmod(String selectStatement) throws SQLException {
        return selectRSmod(selectStatement, null, false);
    }
    
    public ResultSet selectRSmod(String selectStatement, Object param) throws SQLException  {
        return selectRSmod(selectStatement, new Object[]{param}, false);
    }
    
    public ResultSet selectRSmod(String selectStatement, List params) throws SQLException  {
        return selectRSmod(selectStatement, params.toArray(), false);
    }
    
    public ResultSet selectRSmod(String selectStatement, Object[] params) throws SQLException {
        return selectRSmod(selectStatement, params, false);
    }
    
    public ResultSet selectRSlock(String selectStatement) throws SQLException {
        return selectRSmod(selectStatement, null, true);
    }
    
    public ResultSet selectRSlock(String selectStatement, Object param) throws SQLException  {
        return selectRSmod(selectStatement, new Object[]{param}, true);
    }
    
    public ResultSet selectRSlock(String selectStatement, List params) throws SQLException  {
        return selectRSmod(selectStatement, params.toArray(), true);
    }
    
    public ResultSet selectRSlock(String selectStatement, Object[] params) throws SQLException {
        return selectRSmod(selectStatement, params, true);
    }
    
    public void doExecute(String statement) throws SQLException {
        doExecute(statement, null);
    }
    
    public void doExecute(String statement, Object[] params) throws SQLException {
        Trace.print("execute: " + statement);
        PreparedStatement prepStmt = cache.getPrep(statement);
        setParams(prepStmt, params);
        prepStmt.executeUpdate();
//        prepStmt.close();
    }
    
    public List RStoMapList(ResultSet rs) throws SQLException {
        MapListHandler mlh = new MapListHandler();
        List rows = (List)mlh.handle(rs);
        return rows;
    }
    
    public Map RStoMap(ResultSet rs) throws SQLException {
        MapHandler mh = new MapHandler();
        Map m = (Map)mh.handle(rs);
        return m;
    }
    
    public List selectList(String selectStatement) throws SQLException {
        return selectList(selectStatement, null);
    }
    
    public List selectList(String selectStatement, Object[] params) throws SQLException {
        ResultSet rs = selectRS(selectStatement, params);
        List aList = RStoMapList(rs);
        rs.close();
        return aList;
    }
    
    public Map selectOne(String selectStatement) throws SQLException {
        return selectOne(selectStatement, null);
    }
    
    public Map selectOne(String selectStatement, Object[] params) throws SQLException {
        ResultSet rs = selectRS(selectStatement, params);
        List aList = RStoMapList(rs);
        rs.close();
        if (aList.size() > 0) {
            return (Map)aList.get(0);
        } else {
            return null;
        }
    }
    
    public String HTMLmenu(String name, String statement, String defaultv) throws SQLException {
        return HTMLmenu(name, statement, defaultv, null);
    }
    
    public String HTMLmenu(String name, String statement, String defaultv, Object[] params) throws SQLException {
        Trace.print("HTMLmenu: " + statement);
        ResultSet rs = selectRS(statement, params);
        StringBuffer menu = new StringBuffer();
        
        menu.append("<select name='" + name + "'>\n");
        
        while (rs.next()) {
            menu.append("<option value='" + rs.getString(1) + "'");
            if (rs.getString(1).equalsIgnoreCase(defaultv)) {
                menu.append(" selected");
            }
            menu.append(">" + rs.getString(2) + "</option>\n");
        }
        rs.close();
        menu.append("</select>\n");
        return menu.toString();
    }
    
    /*
     *Fixes the problem documented at http://www.macromedia.com/cfusion/knowledgebase/index.cfm?id=tn_18209
     *The Oracle thin driver will only create an updatable resultset if all column names are given explicitly.
     */
    static public String genColNames(String tableName, String[] columnNames) {
        StringBuffer s = new StringBuffer(100);
        for (int i=0; i < columnNames.length; i++) {
            if (i > 0) {
                s.append(", ");
            }
            s.append(tableName + "." + columnNames[i]);
        }
        return s.toString();
    }
    
    /** Getter for property conn.
     * @return Value of property conn.
     *
     */
    public java.sql.Connection getConn() {
        return conn;
    }
    
    /** Setter for property conn.
     * @param conn New value of property conn.
     *
     */
    protected void setConn(java.sql.Connection conn) {
        this.conn = conn;
    }
    
    public String stringParam(String str, int maxLength) {
        if (str.length() > maxLength) {
            return str.substring(0, maxLength);
        } else {
            return str;
        }
    }
    
    public Integer intParam(int val) {
        return Integer.valueOf(val);
    }
    
    public Integer strintParam(String str) {
        return Integer.valueOf(str);
    }
    
    public String clobParam(String str) {
        return str;
    }
    
    public java.sql.Date dateParam(String str) throws ParseException {
        return (new DateFormatter(str)).toDate();
    }
    
    public java.sql.Date dateParam(java.util.Date date) {
        return new java.sql.Date(date.getTime());
    }
    
    public java.sql.Time timeParam(java.util.Date date) {
        return new java.sql.Time(date.getTime());
    }
    
    public java.sql.Timestamp timeStampParam(java.util.Date date) {
        return new java.sql.Timestamp(date.getTime());
    }
    
    public int num2int(Object obj) throws SQLException {
        if (obj.getClass().isInstance(new BigDecimal(0))) {
            return ((BigDecimal)obj).intValue();
        } else if (obj.getClass().isInstance(new Integer(0))) {
            return ((Integer)obj).intValue();
        } else {
            throw new SQLException("Numeric is neither BigDecimal nor Integer");
        }
    }
    
    public void setClobString(ResultSet rs, String field, String str) throws SQLException {
        Clob clob = rs.getClob(field);
        if (clob == null) {
            clob = newClob();
        } else {
            string2Clob(clob, str);
        }
        rs.updateClob(field, clob);
    }
    
    public void setClobString(ResultSet rs, int field, String str) throws SQLException {
        Clob clob = rs.getClob(field);
        if (clob == null) {
            clob = newClob();
        } else {
            string2Clob(clob, str);
        }
        rs.updateClob(field, clob);
    }
    
    public String getClobString(ResultSet rs, String field) throws SQLException {
        Clob clob = rs.getClob(field);
        String str = clob2String(clob);
        return str;
    }
    
    public String getClobString(ResultSet rs, int field) throws SQLException {
        Clob clob = rs.getClob(field);
        String str = clob2String(clob);
        return str;
    }
    
    private void string2Clob(Clob clob, String str) throws SQLException {
        clob.truncate(0L);
        clob.setString(1L, str);
//        try {
//            Writer w = ((oracle.sql.CLOB)clob).getCharacterOutputStream();
//            w.write(str);
//            w.close();
//        } catch (IOException e) {
//            throw new SQLException("Error writing to clob: " + e.getMessage());
//        }
    }
    
    public Clob newClob() throws SQLException {
//        Clob clob = oracle.sql.CLOB.createTemporary(conn, false, oracle.sql.CLOB.DURATION_SESSION);
        Clob clob = oracle.sql.CLOB.getEmptyCLOB();
        return clob;
    }
    
//    public Clob newClob(String str) throws SQLException {
//        Clob clob = newClob();
////        string2Clob(clob, str);
//        return clob;
//    }
    
    private String clob2String(Clob clob) throws SQLException {
        String str;
        if (clob == null) {
            return "";
        }
        long length = clob.length();
        if (length == 0) {
            str = "";
        } else if (length > Integer.MAX_VALUE) {
            throw new SQLException("Length of CLOB too big to be a String");
        } else {
            str = clob.getSubString(1L, (int)clob.length());
        }
        
        return str;
//        try {
//            char[] cbuf = new char[512];
//            Reader r = ((oracle.sql.CLOB)clob).getCharacterStream();
//            BufferedReader br = new BufferedReader(r);
//            StringBuilder sb = new StringBuilder();
//            while (br.read(cbuf, 0, 512) > 0) {
//                sb.append(cbuf);
//            }
//            return sb.toString();
//        } catch (IOException e) {
//            throw new SQLException("Error reading from clob: " + e.getMessage());
//        }
    }
    
    private static final String charSet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz012356789";
    private static final int charSetLength = charSet.length();
    private static final Random rand = new Random();
    
    public char randomChar() {
        return charSet.charAt(rand.nextInt(charSetLength));
    }
    
    public String randomKey(int len) {
        StringBuffer key = new StringBuffer(len);
        for (int n=0; n<len; n++) {
            key.append(randomChar());
        }
        if (key.length() != len) {
            throw new IllegalStateException();
        }
        return key.toString();
    }
    
    public String newKey(String table, String keyField, int len) throws SQLException {
        String select = "SELECT " + keyField + " FROM " + table + " WHERE " + keyField + " = ?";
        String key = randomKey(len);
        boolean found;
        do {
            ResultSet rs = this.selectRS(select, new Object[]{
                key
            });
            found = rs.next();
            rs.close();
        } while (found);
        return key;
    }
    
    public boolean validKey(String key, int len) {
        return (key != null && key.length() == len);
    }
}