/*
 * HiberSessionFilter.java
 *
 * Created on 05 June 2006, 14:42
 */

package jim.util.hiber;

import java.io.*;
import javax.servlet.*;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import jim.common.Logger;
import org.apache.commons.logging.Log;

/**
 *
 * @author  briggsj
 * @version
 */

public class HiberSessionFilter implements Filter {
    
    static private Log log = Logger.getLog(HiberSessionFilter.class);
    
    // The filter configuration object we are associated with.  If
    // this value is null, this filter instance is not currently
    // configured.
    private FilterConfig filterConfig = null;
    
    public HiberSessionFilter() {
    }
    
    /**
     *
     * @param request The servlet request we are processing
     * @param result The servlet response we are creating
     * @param chain The filter chain we are processing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain)
            throws IOException, ServletException {
        log.debug("HiberSessionFilter:doFilter()");
        Throwable problem = null;
        try {
            chain.doFilter(request, response);
            HibernateUtil.commitTransaction();
        } catch(IOException t) {
            log.error("IOException thrown during HiberSessionFilter.doFilter", t);
            throw t;
        } catch(ServletException t) {
            log.error("ServletException thrown during HiberSessionFilter.doFilter", t);
            throw t;
        } finally {
            log.debug("Closing Hibernate session");
            HibernateUtil.closeSession();
        }
    }
    
    
    /**
     * Return the filter configuration object for this filter.
     */
    public FilterConfig getFilterConfig() {
        return (this.filterConfig);
    }
    
    
    /**
     * Set the filter configuration object for this filter.
     *
     * @param filterConfig The filter configuration object
     */
    public void setFilterConfig(FilterConfig filterConfig) {
        
        this.filterConfig = filterConfig;
    }
    
    /**
     * Destroy method for this filter
     *
     */
    public void destroy() {
    }
    
    
    /**
     * Init method for this filter
     *
     */
    public void init(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }
    
    /**
     * Return a String representation of this object.
     */
    public String toString() {
        if (filterConfig == null) return ("HiberSessionFilter()");
        StringBuffer sb = new StringBuffer("HiberSessionFilter(");
        sb.append(filterConfig);
        sb.append(")");
        return (sb.toString());
    }
    
}
