
package jim.util.hiber ;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Clob;
import java.util.Properties;
import jim.common.Logger;
import org.apache.commons.logging.Log;
import org.hibernate.Hibernate;

import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.hibernate.classic.Session;

/**
 * @netbeans.hibernate.util
 */
public class HibernateUtil {
    private static Log log = Logger.getLog(HibernateUtil.class);
    
    private static SessionFactory sessionFactory;
    
    private static final ThreadLocal<Session> threadSession = new ThreadLocal<Session>();
    private static final ThreadLocal<Transaction> threadTransaction = new ThreadLocal<Transaction>();
    
    static {
        try {
            log.debug("Initialising HibernateUtil");
            Configuration cfg = new Configuration();
//            try {
//                readResourcesFromFile(cfg, "/hibernateBeans.lst");
//            } catch (Exception ex) {
//                log.info("No hibernateBeans.lst file");
//            }
//            try {
//                readPropertiesFile(cfg, "/hibernate.properties");
//            } catch (Exception ex) {
//                log.info("No hibernate.properties file");
//            }
//            try {
//                readPropertiesFromXML(cfg, "/hibernate.cfg.xml");
//            } catch (Exception ex) {
//                log.info("No hibernate.cfg.xml file");
//            }
            sessionFactory = cfg.configure().buildSessionFactory();
        } catch (Throwable ex) {
            log.error("Initial SessionFactory creation failed.", ex);
            throw new ExceptionInInitializerError(ex);
        }
    }
    
    private static void readPropertiesFile(Configuration cfg, String filename) throws Exception {
        Properties properties = new Properties();
        properties.load(HibernateUtil.class.getResourceAsStream(filename));
        cfg.setProperties(properties);
    }
    
    private static void readPropertiesFromXML(Configuration cfg, String filename) throws Exception {
        InputStream is = HibernateUtil.class.getResourceAsStream(filename);
        cfg.addInputStream(is);
    }
    
    private static void readResourcesFromFile(Configuration cfg, String filename) throws Exception {
        InputStream is = HibernateUtil.class.getResourceAsStream(filename);
        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
        String line ;
        while ((line = reader.readLine()) != null) {
            cfg.addResource(line);
        }
    }
    
    public static Session getSession() throws HibernateException {
        Session s = (Session) threadSession.get();
        if (s == null) {
            s = sessionFactory.openSession();
            threadSession.set(s);
        }
        return s;
    }
    
    public static void closeSession() throws HibernateException {
        Session s = (Session) threadSession.get();
        threadSession.set(null);
        if (s != null && s.isOpen()) {
            s.close();
            threadSession.set(null);
        }
    }
    
    public static void beginTransaction() {
        Transaction tx = (Transaction)threadTransaction.get();
        if (tx == null) {
            tx = getSession().beginTransaction();
            threadTransaction.set(tx);
        }
    }
    
    public static void commitTransaction() {
        Transaction tx = (Transaction)threadTransaction.get();
        if (tx != null && !tx.wasCommitted() && !tx.wasRolledBack()) {
            tx.commit();
            threadTransaction.set(null);
        }
        
    }
    
    public static void rollbackTransaction() {
        Transaction tx = (Transaction)threadTransaction.get();
        if (tx != null && !tx.wasCommitted() && !tx.wasRolledBack()) {
            tx.rollback();
            threadTransaction.set(null);
            closeSession();
        }
    }
    
    /**
     * Getter for property sessionFactory.
     * @return Value of property sessionFactory.
     */
    public static SessionFactory getSessionFactory() {
        getSession();
        return sessionFactory;
    }
    
    static public Clob createClob (String s) {
        if (s == null) {
            return Hibernate.createClob("");
        } else {
            return Hibernate.createClob(s);
        }
    }
}